@@ -1,6 +1,6 @@
 #!/usr/bin/perl
 # -------------------------------------------------------
-# $Header: /home/cvs/dobackup/dobackup.pl,v 4.28 2004/01/09 23:16:52 imorgan Exp $
+# $Header: /home/cvs/dobackup/dobackup.pl,v 4.30 2004/03/05 15:56:13 imorgan Exp $
 # -------------------------------------------------------
 #
 # Copyright 1997-2004 Webcon, Inc.
@@ -205,7 +205,7 @@ sub ShowSyntax {
   my($temp)=('');
 
   print "\nUsage: $ExecutableName <options>\n";
-  $temp = '$Revision: 4.28 $';
+  $temp = '$Revision: 4.30 $';
   ($temp) = ( $temp =~ /^\$Revision\:\ (.*?)\ \$$/ );
   print 'Version: ',"$temp\n";
   print <<EOF;
@@ -733,8 +733,9 @@ sub PerformBackup{
   my($SeekTo);
   my($Return)=0;
   my($Filename)='';
-  my($Usage);
-  my($Stamp);
+  my($Usage)=0;
+  my($UsageExact)=0;
+  my($Stamp)=0;
 
   if ( $opt_fullbackup || $Timestamp{$Host} == 0 ) { #Full Backup
     $Debug && print ("Performing Full Backup!\n");
@@ -910,7 +911,7 @@ sub PerformBackup{
       $Temp=$IniConfig->val($Host,'MediaSet');
       $Temp=$IniConfig->val("MediaSet=$Temp",'CurrentUnit');
       $IniConfig->setval("MediaUnit=$Temp",'Usage',$Usage);
-      $IniConfig->WriteConfig($ConfigFile);
+      $IniConfig->RewriteConfig;
     }
     elsif ($Return == 0xff00 ) {
        print("ERROR: BackupTimeStamp $MT tell command failed!\n\nAborting backup!\n");
@@ -935,7 +936,7 @@ sub PerformBackup{
 
         $Debug && print("Since the writing of the Backup location failed, the next backup will be a _FULL_ backup!\n");
         $IniConfig->setval($Host,'Timestamp',0); #write a zero timestamp to force full backup
-        $IniConfig->WriteConfig($ConfigFile);
+        $IniConfig->RewriteConfig;
         return;
     } else {
       print "ERROR: BackupTimeStamp $MT tell ran with";
@@ -953,12 +954,16 @@ sub PerformBackup{
     $Return = 0xffff & system("$DF -h $Destination{$Host} |$Tail -1 |$TR -s ' ' |$Cut -f 4 -d ' ' > $BackupTimeStamp");
     if ($Return == 0) {
       chomp($Usage = `$Cat $BackupTimeStamp`);
+      $Debug && print "Command: $DF -B 1 $Destination{$Host} |$Tail -1 |$TR -s ' ' |$Cut -f 4 -d ' ' > $BackupTimeStamp\n";
+      $Return = 0xffff & system("$DF -B 1 $Destination{$Host} |$Tail -1 |$TR -s ' ' |$Cut -f 4 -d ' ' > $BackupTimeStamp");
+      chomp($UsageExact = `$Cat $BackupTimeStamp`);
       unlink("$BackupTimeStamp");
-      $Debug && print "Usage=$Usage\n";
+      $Debug && print "Usage=$Usage ($UsageExact)\n";
       $Temp=$IniConfig->val($Host,'MediaSet');
       $Temp=$IniConfig->val("MediaSet=$Temp",'CurrentUnit');
       $IniConfig->setval("MediaUnit=$Temp",'Usage',$Usage);
-      $IniConfig->WriteConfig($ConfigFile);
+      $IniConfig->setval("MediaUnit=$Temp",'UsageExact',$UsageExact);
+      $IniConfig->RewriteConfig;
     }
     elsif ($Return == 0xff00 ) {
        print("ERROR: BackupTimeStamp $DF command failed!\n\nAborting backup!\n");
@@ -969,7 +974,7 @@ sub PerformBackup{
 
        print("Since the writing of the DF output failed, the next backup will be a _FULL_ backup!\n");
        $IniConfig->setval($Host,'Timestamp',0); #write a zero timestamp to force full backup
-       $IniConfig->WriteConfig($ConfigFile);
+       $IniConfig->RewriteConfig;
        return;
     } else {
       print "ERROR: BackupTimeStamp $DF ran with";
@@ -999,6 +1004,13 @@ sub PerformBackup{
 
 sub Cleanup{
   my($temp)=0;
+  my($temp2)=0;
+  my($Usage)=0;
+  my($UsageExact)=0;
+  my($LastSize)=0;
+  my($LastSizeBytes)=0;
+  my($LastSizeFull)=0;
+
   system("$MV $ToBackupTempFile $BackupLogDir/$BackupDay.log.$Host");
 
   # Mail off the results
@@ -1006,7 +1018,7 @@ sub Cleanup{
   open (MAILTHIS, "|$MailProg $BackupAdministrator");
   print MAILTHIS "To: $BackupAdministrator\nSubject: $Host ",($Debug?"Debug-mode":"Automated")," Backup Summary\n\n";
 
-  $temp = '$Revision: 4.28 $';
+  $temp = '$Revision: 4.30 $';
   ($temp) = ( $temp =~ /^\$Revision\:\ (.*?)\ \$$/ );
 
   print MAILTHIS "This is an automated backup report from $ExecutableName v$temp\n\n";
@@ -1025,14 +1037,17 @@ sub Cleanup{
     $temp .= ".debug" if ($Debug);
 
     print MAILTHIS "File:            $temp\n";
-    $temp = `$LS -s -h $temp`;
-    ($temp) = ( $temp =~ /^\s*(.*?)\s.*$/ );
-    $temp =~ s/\b(\d+)\w*[kK]\b/$1 KiB/g;
-    $temp =~ s/\b(\d+)\w*M\b/$1 MiB/g;
-    $temp =~ s/\b(\d+)\w*G\b/$1 GiB/g;
 
-    print MAILTHIS "Size:            $temp\n";
-    $Debug && print "Size: $temp\n";
+    $LastSizeBytes = `$LS -s --block-size=1 $temp`;
+    ($LastSizeBytes) = ( $LastSizeBytes =~ /^\s*(\d*?)\s.*$/ );
+    $LastSize = `$LS -s -h $temp`;
+    ($LastSize) = ( $LastSize =~ /^\s*(.*?)\s.*$/ );
+    $LastSize =~ s/\b(\d+)\w*[kK]\b/$1 KiB/g;
+    $LastSize =~ s/\b(\d+)\w*M\b/$1 MiB/g;
+    $LastSize =~ s/\b(\d+)\w*G\b/$1 GiB/g;
+
+    print MAILTHIS "Size:            $LastSize\n";
+    $Debug && print "Size: $LastSize ($LastSizeBytes)\n";
   }
   elsif ($Mode{$Host} eq 'Tape') {
     print MAILTHIS "Device:          $Destination{$Host}\n";
@@ -1053,12 +1068,37 @@ sub Cleanup{
 
   $temp = $IniConfig->val("MediaSet=$MediaSet{$Host}",'CurrentUnit');
   print MAILTHIS "Current Unit:    [$MediaSet{$Host}][$temp]\n";
-  $temp = $IniConfig->val("MediaUnit=$temp",'Usage');
-  $temp =~ s/\b(\d+)\w*[kK]\b/$1 KiB/g;
-  $temp =~ s/\b(\d+)\w*M\b/$1 MiB/g;
-  $temp =~ s/\b(\d+)\w*G\b/$1 GiB/g;
-  ($Mode{$Host} eq 'Tape') && print MAILTHIS 'Current Tape Position: ' , $temp , "\n";
-  ($Mode{$Host} eq 'Disk' || $Mode{$Host} eq 'Filesystem') && print MAILTHIS 'Space Remaining: ' , $temp , "\n";
+  $Usage = $IniConfig->val("MediaUnit=$temp",'Usage');
+  $Usage =~ s/\b(\d+)\w*[kK]\b/$1 KiB/g;
+  $Usage =~ s/\b(\d+)\w*M\b/$1 MiB/g;
+  $Usage =~ s/\b(\d+)\w*G\b/$1 GiB/g;
+  ($Mode{$Host} eq 'Tape') && print MAILTHIS 'Current Tape Position: ' , $Usage , "\n";
+  ($Mode{$Host} eq 'Disk' || $Mode{$Host} eq 'Filesystem') && print MAILTHIS 'Space Remaining: ' , $Usage , "\n";
+
+  # write LastSizeBytes to lastsizefull if in full mode
+  if ($opt_fullbackup || $Timestamp{$Host} == 0) {
+    $IniConfig->setval("$Host",'LastSizeFull',"$LastSizeBytes");
+  }
+  $IniConfig->setval("$Host",'LastSize',"$LastSizeBytes");
+  $IniConfig->RewriteConfig;
+
+  $temp = $IniConfig->val("MediaSet=$MediaSet{$Host}",'CurrentUnit');
+  $UsageExact = $IniConfig->val("MediaUnit=$temp",'UsageExact');
+  $LastSizeFull = $IniConfig->val("$Host",'LastSizeFull');
+  $LastSizeBytes = $IniConfig->val("$Host",'LastSize');
+  $Debug && print "Free: $UsageExact\n";
+  $Debug && print "Full: $LastSizeFull\n";
+  $Debug && print "Last: $LastSizeBytes\n";
+
+  if ( $UsageExact < $LastSizeFull ) {
+   $Debug && print "WARNING: free space may be to low for next FULL backup\n";
+   print MAILTHIS "\nWARNING: Space Remaining may be too low for the next FULL backup to complete!\n";
+  }
+  if ( $UsageExact < $LastSizeBytes ) {
+   $Debug && print "WARNING: free space may be to low for next backup\n";
+   print MAILTHIS "\nWARNING: Space Remaining may be too low for the next regular backup to complete!\n";
+  }
+
   close(MAILTHIS);
 }
 # ------------------- End Cleanup
@@ -1122,6 +1162,7 @@ sub LoadConfig {
     # Backward-compat: if missing, assume true.
     if ( ! defined($Auto{$HostName}) ) {
       $Auto{$HostName}      = '1';
+      $IniConfig->newval($HostName,'Auto','1');
     }
 
     $IP{$HostName}          = $IniConfig->val($HostName,'IP');
@@ -1129,20 +1170,40 @@ sub LoadConfig {
     # Backward-compat: if missing, assume normal.
     if ( ! defined($IncrMode{$HostName}) ) {
       $IncrMode{$HostName}  = 'Normal';
+      $IniConfig->newval($HostName,'IncrementalMode','Normal');
     }
 
     $Timestamp{$HostName}   = $IniConfig->val($HostName,'Timestamp');
     $TimestampFull{$HostName}   = $IniConfig->val($HostName,'TimestampFull');
+    # Backward-compat: if missing, assume last timestamp.
     if ( ! defined($TimestampFull{$HostName}) ) {
       $TimestampFull{$HostName} = $Timestamp{$HostName};
+      $IniConfig->newval($HostName,'TimestampFull',$Timestamp{$HostName});
     }
 
-    $temp= $IniConfig->val($HostName,'MediaSet');
+    $temp = $IniConfig->val($HostName,'LastSize');
+    # Backward-compat: if missing, assume 0.
+    if ( ! defined($temp) ) {
+      $IniConfig->newval($HostName,'LastSize',0);
+    }
+
+    $temp = $IniConfig->val($HostName,'LastSizeFull');
+    # Backward-compat: if missing, assume lastsize.
+    if ( ! defined($temp) ) {
+      $IniConfig->newval($HostName,'LastSizeFull',$IniConfig->val($HostName,'LastSize'));
+    }
+
+    $temp = $IniConfig->val($HostName,'MediaSet');
     $MediaSet{$HostName}    = $temp;
     $Mode{$HostName}        = $IniConfig->val("MediaSet=$MediaSet{$HostName}",'Mode');
     $Destination{$HostName} = $IniConfig->val("MediaSet=$MediaSet{$HostName}",'Destination');
     $MediaUnit{$HostName}   = $IniConfig->val("MediaSet=$MediaSet{$HostName}",'CurrentUnit');
 
+    $temp = $IniConfig->val("MediaUnit=$MediaUnit{$HostName}",'UsageExact');
+    if ( ! defined($temp)) {
+      $IniConfig->newval("MediaUnit=$MediaUnit{$HostName}",'UsageExact','0');
+    }
+
     $Paths{$HostName}       = [ $IniConfig->val($HostName,'Paths') ];
     $Excludes{$HostName}    = [ $IniConfig->val($HostName,'Excludes') ];
 
@@ -1150,7 +1211,7 @@ sub LoadConfig {
       print "ERROR: Invalid IP address ($IP{$HostName}) for host $HostName.\n";
       exit 1;
     }
-    elsif ($1 < 0 or $1 > 255 or $2 < 0 or $2 > 255 or $3 < 0 or $3 > 255 or $4 < 1 or $4 > 254) {
+    elsif ($1 < 1 or $1 > 254 or $2 < 0 or $2 > 255 or $3 < 0 or $3 > 255 or $4 < 1 or $4 > 254) {
       print "ERROR: Invalid IP address ($IP{$HostName}) for host $HostName.\n";
       exit 1;
     }
@@ -1168,6 +1229,8 @@ sub LoadConfig {
     $Debug && print "TimestampFull=",$TimestampFull{$HostName},"\n";
     $Debug && print "Mode=",$Mode{$HostName},"\n";
     $Debug && print "Destination=",$Destination{$HostName},"\n";
+    $Debug && print "MediaSet=",$MediaSet{$HostName},"\n";
+    $Debug && print "CurrentUnit=",$MediaUnit{$HostName},"\n";
     $Debug && print "Paths=<<EOP\n";
     foreach (@{$Paths{$HostName}}) {
       $Debug && print "$_\n";
@@ -1197,7 +1260,7 @@ sub UpdateConfig {
   if ( $opt_fullbackup || $Timestamp{$Host} == 0 ) {
     $IniConfig->setval($Host,'TimestampFull',$StartTime);
   }
-  $IniConfig->WriteConfig($ConfigFile);
+  $IniConfig->RewriteConfig;
   $Debug && print "Config file backup timestamp updated for host $Host.\n";
 }
 
@@ -1282,7 +1345,7 @@ sub RotateMedia {
       }
     }
   }
-  $IniConfig->WriteConfig($ConfigFile);
+  $IniConfig->RewriteConfig;
   $Debug && print "Wrote updated config file.\n";
 
   if ($actions ne '') {
@@ -1370,7 +1433,7 @@ sub PrepDisk {
     $Debug && print "Resetting NeedsPrep flag for $MediaSet{$Host}.\n";
     $IniConfig->setval("MediaSet=$MediaSet{$Host}",'NeedsPrep',"0");
     $Debug && print "Writing config file.\n";
-    $IniConfig->WriteConfig($ConfigFile);
+    $IniConfig->RewriteConfig;
     return 0; # prep sucessful.
   } else {
     $Debug && print "Disk does need not need prepping.\n";
